unit uServerContainer;

interface

uses System.SysUtils, System.Classes,
  Datasnap.DSTCPServerTransport,
  Datasnap.DSServer, Datasnap.DSCommonServer,
  Datasnap.DSAuth, IndyPeerImpl, System.Generics.Collections,
  Data.SqlExpr, Datasnap.DSService, Vcl.Forms, Data.DB, Datasnap.DBClient;

type
  TServerContainer = class(TDataModule)
    DSServer: TDSServer;
    DSTCPServerTransport: TDSTCPServerTransport;
    scGlobal: TDSServerClass;
    cdsConAtivas: TClientDataSet;
    cdsConAtivasSessao: TLargeintField;
    cdsConAtivasBanco: TStringField;
    cdsConAtivasUsuario: TStringField;
    procedure scGlobalGetClass(DSServerClass: TDSServerClass;
      var PersistentClass: TPersistentClass);
    procedure DataModuleCreate(Sender: TObject);
    procedure DSServerDisconnect(DSConnectEventObject: TDSConnectEventObject);
    procedure DataModuleDestroy(Sender: TObject);
  private
    { Private declarations }
    ListofConnection : TDictionary<NativeInt, TSQLConnection>;
    ListOfUsers : TDictionary<NativeInt, string>;
  public
    ConfigServer : String;
    function CreateConnection(DataBase : String) : Boolean;
    function GetConnection : TSQLConnection;
    function CreateUserData(User : String) : Boolean;
  end;

var
  ServerContainer: TServerContainer;

implementation

uses Winapi.Windows, usmGlobal;

{$R *.dfm}

function TServerContainer.CreateConnection(DataBase: String): Boolean;
var
  dbconn : TSQLConnection;
begin
  Try
    Result := True;
    dbconn := TSQLConnection.Create(Self);
    dbconn.DriverName := 'MYSQL';
    dbconn.ConnectionName := 'Cliente';
    dbconn.LoadParamsFromIniFile(ConfigServer); // Caminho e nome do Arquivo de configurao 'DbConfig.ini'
    dbconn.Params.Values['Database'] := DataBase;
    dbconn.LoginPrompt := False;
    dbconn.Open;
    ListofConnection.Add(TDSSessionManager.GetThreadSession.Id, dbconn);
    cdsConAtivas.AppendRecord([TDSSessionManager.GetThreadSession.Id, DataBase, '']) ;
  Except     // Declarao uses: Datasnap.DSService
    Result := False;
  End;
end;

function TServerContainer.CreateUserData(User: String): Boolean;
begin
  Result := True;
  Try
    ListOfUsers.Add(TDSSessionManager.GetThreadSession.Id, User);
    If cdsConAtivas.FindKey([TDSSessionManager.GetThreadSession.Id]) Then Begin
      cdsConAtivas.Edit;
      cdsConAtivasUsuario.AsString := User;
      cdsConAtivas.Post;
    End;
  Except
    Result := False;
  End;
end;

procedure TServerContainer.DataModuleCreate(Sender: TObject);
begin
  ListofConnection := TDictionary<NativeInt, TSQLConnection>.Create;
  ListOfUsers := TDictionary<NativeInt, string>.Create;
  ConfigServer := ExtractFilePath(Application.ExeName) + 'ConfigDB.ini';
end;

procedure TServerContainer.DataModuleDestroy(Sender: TObject);
begin
  FreeAndNil(ListofConnection);
  FreeAndNil(ListOfUsers);
end;

procedure TServerContainer.DSServerDisconnect(
  DSConnectEventObject: TDSConnectEventObject);
begin
  If GetConnection <> nil Then Begin
    GetConnection.Close;
    GetConnection.Free;
    ListofConnection.Remove(TDSSessionManager.GetThreadSession.Id);
    ListOfUsers.Remove(TDSSessionManager.GetThreadSession.Id);
    If cdsConAtivas.FindKey([TDSSessionManager.GetThreadSession.Id]) Then
      cdsConAtivas.Delete;
  End;
end;

function TServerContainer.GetConnection: TSQLConnection;
var
  dbconn : TSQLConnection;
begin
  if ListofConnection.ContainsKey(TDSSessionManager.GetThreadSession.Id) then
    Result := ListofConnection[TDSSessionManager.GetThreadSession.Id]
  else
    If CreateConnection('cliente') Then // cliente banco de dados modelo (teste)
      Result := ListofConnection[TDSSessionManager.GetThreadSession.Id]
                 // Aqui eu uso para informar a conexo durante a criao da aplicao cliente.
    Else
      Result := nil;
end;

procedure TServerContainer.scGlobalGetClass(
  DSServerClass: TDSServerClass; var PersistentClass: TPersistentClass);
begin
  PersistentClass := usmGlobal.TsmGlobal;
end;


end.

